/*
 * 	Doku, siehe http://www.mikrocontroller.net/articles/AVR_FAT32
 *  Neuste Version: http://www.mikrocontroller.net/svnbrowser/avr-fat32/
 *	Autor: Daniel R.
 */

#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/io.h>

#include "clksys_driver.h"
#include "config.h"	// Hier werden alle nötigen Konfigurationen vorgenommen !
#include "file.h"
#include "fat.h"
#include "mmc.h"		// Hardware abhngig
#include "uart.h"		// Hardware abhngig, es kann auch eine eigene eingebunden werden !

/* The LED to use for visual feedback. */
#define LEDPORT PORTE
#define LEDMASK 0xFF
//*****************************************************************************************************************


void main(void){

		/*  Enable internal 32 MHz ring oscillator and wait until it's
		 *  stable. Divide clock by two with the prescaler C and set the
		 *  32 MHz ring oscillator as the main clock source. Wait for
		 *  user input while the LEDs toggle.
		 */
		CLKSYS_Enable( OSC_RC32MEN_bm );
		CLKSYS_Prescalers_Config( CLK_PSADIV_1_gc, CLK_PSBCDIV_1_1_gc );
		do {} while ( CLKSYS_IsReady( OSC_RC32MRDY_bm ) == 0 );
		do {} while ( CLKSYS_IsReady( OSC_RC32MRDY_bm ) == 0 );
		CLKSYS_Main_ClockSource_Select( CLK_SCLKSEL_RC32M_gc );
    CLKSYS_Disable(OSC_RC2MEN_bm);



	/* Set up user interface. */
	LEDPORT.DIRSET = LEDMASK;
	LEDPORT.OUTSET = LEDMASK;

	/* Set up Timer/Counter 0 to work from CPUCLK/64, with period 10000 and
	 * enable overflow interrupt.
	 */
	TCC0.PER = 10000;
	TCC0.CTRLA = ( TCC0.CTRLA & ~TC0_CLKSEL_gm ) | TC_CLKSEL_DIV64_gc;
	TCC0.INTCTRLA = ( TCC0.INTCTRLA & ~TC0_OVFINTLVL_gm ) |
	                TC_OVFINTLVL_MED_gc;

 	/* Enable low interrupt level in PMIC and enable global interrupts. */
	PMIC.CTRL |= PMIC_MEDLVLEN_bm;
	sei();


  

	// Uart initialisierung zu Ausgabe von Daten
	uinit();

	uputs((unsigned char*)"\nBoot");

	// Versuch Karte zu Initialisieren, bis es klappt. Umbedingt so, weil die Initialiesierung nicht immer auf anhieb klappt.
	while (FALSE==mmc_init()){
		nop();
	}

	uputs((unsigned char*)"...");


	// Fat initialisieren. Nur wenn das klappt sind weitere Aktionen sinnvoll, sonst aendet das Programm !
	if(TRUE == fat_loadFatData()){

	  uputs((unsigned char*)"Ok\n");		// Wenn auf dem terminal "Boot... OK" zu lesen ist, ist Init OK. Jetzt kann man schreiben/anhaengen/lesen



	  // Dateinamen muessen in diesem Format sein ! Man beachte die Gre des Arrays und die Grobuchstaben!
	  unsigned char file_name[13]="TEST    TXT";

	  // String zum in die Datei schreiben.
	  unsigned char str[14]="Hallo Datei!";






	  // Datei existiert nicht, also anlegen !
	  if(MMC_FILE_NEW == ffopen(file_name)){

	     // Schreibt String auf Karte ! Nur richtige Strings koennen mit ffwrites geschrieben werden !
	     ffwrites(str);

	     // Neue Zeile in der Datei
	     ffwrite(0x0D);
	     ffwrite(0x0A);

	     // Schliet Datei
	     ffclose();
	   }




	   // Datei existiert, also anhaengen !
	  if(MMC_FILE_EXISTS == ffopen(file_name)){

	     ffseek(file.length);   // Spult bis zum Dateiende vor um anzuhaengen, geht auch ohne Option MMC_OVER_WRITE !

	     // schreibt String
	     //ffwrites(str);

	     // neue Zeile in der Datei
	     //ffwrite(0x0D);
	     //ffwrite(0x0A);

	     // schliet Datei
	     ffclose();
	   }




	   // Datei existiert also lesen. Gerade angelegt und beschrieben, oder war schon vorhanden und es wurde was angehaengt.
	   if(MMC_FILE_EXISTS == ffopen(file_name)){

	      // setzen einer Variable und dann runterzaehlen geht am schnellsten !
	      unsigned long int seek=file.length;

	      // lesen eines chars und Ausgabe des chars.
	      // solange bis komplette Datei gelesen wurde.
	      do{
	         uputc(ffread());
	      }while(--seek);

	      ffclose();
	   }

      for(;;);


	  }

}

/*! Just toggle LED(s) when interrupt occurs. */
ISR(TCC0_OVF_vect)
{
	LEDPORT.OUTTGL = LEDMASK;
}
