/*
 * 	Doku, siehe http://www.mikrocontroller.net/articles/AVR_FAT32
 *  Neuste Version: http://www.mikrocontroller.net/svnbrowser/avr-fat32/
 *	Autor: Daniel R.
 */



#include "uart.h"
#include "config.h"


void uinit(void)
{
	/* This PORT setting is only valid to USARTD0 if other USARTs is used a
	 * different PORT and/or pins is used. */
	/* PIN3 (TXD0) as output. */
	PORTD.DIRSET = PIN3_bm;

	/* PIN2 (RXD0) as input. */
	PORTD.DIRCLR = PIN2_bm;

	/* USARTC0, 8 Data bits, No Parity, 1 Stop bit. */
	USART_Format_Set(&USART, USART_CHSIZE_8BIT_gc, USART_PMODE_DISABLED_gc, false);

	/* Set Baudrate to 9600 bps:
	 * Use the default I/O clock fequency that is 2 MHz.
	 * Do not use the baudrate scale factor
	 *
	 * Baudrate select = (1/(16*(((I/O clock frequency)/Baudrate)-1)
	 *                 = 12
   * Baudrate for 16MHz and 115200bps = 983 and 0b1001
   * Baudrate select for 32MHz and 115200bps 1047 and 0b1010
	 */
	USART_Baudrate_Set(&USART, 1047 , 0b1010);

	/* Enable both RX and TX. */
	USART_Rx_Enable(&USART);
	USART_Tx_Enable(&USART);
}

/*! \brief putting out a single Byte
 *  This function has to be used to put out a single Byte
 *  
 *  it will idle until the register empty flag occurs
 *  the USART has to be defined before calling this function
 *
 *  \note Use the "#define" instruction for declaring USART, e.g. use
 *        "#define USART USARTC0"
 *
 *  \param sendData        Byte that will be sent
 *
 */
 
void uputc (unsigned char sendData)
{
  do{
  }while(!USART_IsTXDataRegisterEmpty(&USART));
		USART_PutChar(&USART, sendData);
}


/*! \brief putting out a whole String
 *  This function has to be used to put out a whole String
 *  
 *  It calls the uputc function for every valid character of the array
 *  which is pointed by referenced pointer s
 *  It will stop automatically when an invalid char occurs, like the "\0" 
 *  of an array or string
 *
 *
 *  \param s               Pointer to the string that has to be sent
 *
 */
void uputs(unsigned char *s)
{
  while(*s) uputc(*s++);
}


